import { RpcCallShared, RpcCancelFn } from "./rpc-call-shared";
import { IMessageType } from "@protobuf-ts/runtime";
import { RpcOutputStream } from "./rpc-output-stream";
import { RpcStatus } from "./rpc-status";
import { MethodInfo, ServiceInfo } from "./reflection-info";
import { RpcMetadata } from "./rpc-metadata";
/**
 * A server streaming RPC call. The client provides exactly one input message
 * but the server may respond with 0, 1, or more messages.
 */
export declare class ServerStreamingCall<I extends object = object, O extends object = object> implements RpcCallShared<I, O>, PromiseLike<FinishedServerStreamingCall<I, O>> {
    /**
     * Reflection information about the service this call belongs to.
     */
    readonly serviceInfo: ServiceInfo;
    /**
     * Reflection information about this call.
     */
    readonly methodInfo: MethodInfo<I, O>;
    /**
     * The input type of the RPC.
     */
    readonly requestType: IMessageType<I>;
    /**
     * Request headers being sent with the request.
     *
     * Request headers are provided in the `meta` property of the
     * `RpcOptions` passed to a call.
     */
    readonly requestHeaders: Readonly<RpcMetadata>;
    /**
     * The request message being sent.
     */
    readonly requestMessage: Readonly<I>;
    /**
     * The output type of the RPC.
     */
    readonly responseType: IMessageType<O>;
    /**
     * The response headers that the server sent.
     *
     * This promise will reject with a `RpcError` when the server sends a
     * error status code.
     */
    readonly responseHeaders: Promise<RpcMetadata>;
    /**
     * Response messages from the server.
     * This is an AsyncIterable that can be iterated with `await for .. of`.
     */
    readonly responseStream: RpcOutputStream<O>;
    /**
     * The response status the server replied with.
     *
     * This promise will resolve when the server has finished the request
     * successfully.
     *
     * If the server replies with an error status, this promise will
     * reject with a `RpcError`.
     */
    readonly responseStatus: Promise<RpcStatus>;
    /**
     * The trailers the server attached to the response.
     *
     * This promise will resolve when the server has finished the request
     * successfully.
     *
     * If the server replies with an error status, this promise will
     * reject with a `RpcError`.
     */
    readonly responseTrailers: Promise<RpcMetadata>;
    private readonly cancelFn;
    constructor(serviceInfo: ServiceInfo, methodInfo: MethodInfo<I, O>, requestHeaders: Readonly<RpcMetadata>, requestMessage: Readonly<I>, responseHeaders: Promise<RpcMetadata>, responseStream: RpcOutputStream<O>, responseStatus: Promise<RpcStatus>, responseTrailers: Promise<RpcMetadata>, cancelFn: RpcCancelFn);
    /**
     * Cancel this call.
     */
    cancel(): void;
    /**
     * Instead of awaiting the response status and trailers, you can
     * just as well await this call itself to receive the final outcome.
     * You should first setup some listeners to the `requestMessage` to
     * see the actual messages the server replied with.
     */
    then<TResult1 = FinishedServerStreamingCall<I, O>, TResult2 = never>(onfulfilled?: ((value: FinishedServerStreamingCall<I, O>) => (PromiseLike<TResult1> | TResult1)) | undefined | null, onrejected?: ((reason: any) => (PromiseLike<TResult2> | TResult2)) | undefined | null): PromiseLike<TResult1 | TResult2>;
    private promiseFinished;
}
/**
 * A completed server streaming RPC call. The server will not send any more
 * messages.
 */
export interface FinishedServerStreamingCall<I extends object, O extends object> {
    /**
     * Reflection information about the service this call belongs to.
     */
    readonly serviceInfo: ServiceInfo;
    /**
     * Reflection information about this call.
     */
    readonly methodInfo: MethodInfo<I, O>;
    /**
     * The input type of the RPC.
     */
    readonly requestType: IMessageType<I>;
    /**
     * Request headers being sent with the request.
     */
    readonly requestHeaders: Readonly<RpcMetadata>;
    /**
     * The request message being sent.
     */
    readonly requestMessage: Readonly<I>;
    /**
     * The output type of the RPC.
     */
    readonly responseType: IMessageType<O>;
    /**
     * The response headers that the server sent.
     */
    readonly responseHeaders: RpcMetadata;
    /**
     * The response status the server replied with.
     * The status code will always be OK.
     */
    readonly responseStatus: RpcStatus;
    /**
     * The trailers the server attached to the response.
     */
    readonly responseTrailers: RpcMetadata;
}
