import { Deferred } from "./deferred";
/**
 * A `RpcOutputStream` that you control.
 */
export class RpcOutputStreamController {
    constructor() {
        this._onNextListeners = [];
        this._onMessageListeners = [];
        this._onErrorListeners = [];
        this._onCompleteListeners = [];
        this._pendingResult = new Deferred();
        this._closed = false;
    }
    /**
     * Is this stream already closed by a completion or error?
     */
    get closed() {
        return this._closed;
    }
    assertOpen() {
        if (this._closed)
            throw new Error('stream is already closed');
    }
    onNext(callback) {
        this._onNextListeners.push(callback);
        return () => {
            let i = this._onNextListeners.indexOf(callback);
            if (i >= 0)
                this._onNextListeners.splice(i, 1);
        };
    }
    onMessage(callback) {
        this._onMessageListeners.push(callback);
        return () => {
            let i = this._onMessageListeners.indexOf(callback);
            if (i >= 0)
                this._onMessageListeners.splice(i, 1);
        };
    }
    onError(callback) {
        this._onErrorListeners.push(callback);
        return () => {
            let i = this._onErrorListeners.indexOf(callback);
            if (i >= 0)
                this._onErrorListeners.splice(i, 1);
        };
    }
    onComplete(callback) {
        this._onCompleteListeners.push(callback);
        return () => {
            let i = this._onCompleteListeners.indexOf(callback);
            if (i >= 0)
                this._onCompleteListeners.splice(i, 1);
        };
    }
    [Symbol.asyncIterator]() {
        if (!this._pendingResult) {
            return {
                next: () => Promise.resolve({
                    done: true,
                    value: null,
                })
            };
        }
        return {
            next: () => this._pendingResult.promise,
        };
    }
    /**
     * Emits a new message. Throws if stream is already closed.
     */
    notifyMessage(message) {
        this.assertOpen();
        if (this._pendingResult.state === 0 /* PENDING */) {
            let cur = this._pendingResult;
            this._pendingResult = new Deferred();
            cur.resolve({
                value: message,
                done: false,
            });
        }
        for (let i = 0; i < this._onMessageListeners.length; i++)
            this._onMessageListeners[i](message);
        for (let i = 0; i < this._onNextListeners.length; i++)
            this._onNextListeners[i](message, undefined, false);
    }
    /**
     * Closes the stream with an error. Throws if stream is already closed.
     */
    notifyError(error) {
        this.assertOpen();
        this._closed = true;
        this._pendingResult.rejectPending(error);
        for (let i = 0; i < this._onErrorListeners.length; i++)
            this._onErrorListeners[i](error);
        for (let i = 0; i < this._onNextListeners.length; i++)
            this._onNextListeners[i](undefined, error, true);
    }
    /**
     * Closes the stream successfully. Throws if stream is already closed.
     */
    notifyComplete() {
        this.assertOpen();
        this._closed = true;
        this._pendingResult.resolvePending({
            value: null,
            done: true,
        });
        for (let i = 0; i < this._onCompleteListeners.length; i++)
            this._onCompleteListeners[i]();
        for (let i = 0; i < this._onNextListeners.length; i++)
            this._onNextListeners[i](undefined, undefined, true);
        this._onErrorListeners.splice(0, this._onErrorListeners.length);
        this._onMessageListeners.splice(0, this._onMessageListeners.length);
        this._onCompleteListeners.splice(0, this._onCompleteListeners.length);
    }
}
//# sourceMappingURL=rpc-output-stream.js.map